
MODULE Speedup;

FROM SYSTEM        IMPORT NULL, ADDRESS;
FROM Ports         IMPORT MsgPortPtr;
FROM PortUtils     IMPORT CreatePort, CreateStdIO, DeleteStdIO,
                          DeletePort;
FROM IO            IMPORT IOStdReqPtr, IORequestPtr, IORequest;
FROM Devices       IMPORT OpenDevice, CloseDevice;
FROM CommandLine   IMPORT GetCL, CLStrings;
FROM InOut         IMPORT WriteLn, WriteString;
FROM LongInOut     IMPORT WriteLongCard;
FROM Strings       IMPORT Length, Compare, CompareResults, Pos;
FROM M2Conversions IMPORT ConvertToCardinal;

 
VAR argc, i, Where : CARDINAL;
    args : ARRAY[0..8] OF CLStrings;
    diskport : MsgPortPtr;
    diskreq  : IOStdReqPtr;
    open, ticks, drive: LONGCARD;
    ioUnitStuffPtr : ADDRESS;
    TrackTicks: POINTER TO LONGCARD;
    DontCare, OK: BOOLEAN;
    
PROCEDURE UpperCase(VAR s:ARRAY OF CHAR);
VAR i: CARDINAL;
BEGIN
  FOR i:=0 TO Length(s) DO
    IF (s[i]>='a') AND (s[i]<='z') THEN
      s[i]:=CHR(ORD(s[i])-32);
    END;
  END;
END UpperCase;

PROCEDURE Init() : BOOLEAN;
  (* returns true if Initialization was successful,
     also sets global variable open to 0 on success, or
     error number if OpenDevice failed *)
BEGIN
  diskport:=CreatePort("",0);
  IF diskport=NULL THEN
    RETURN FALSE;
  END;
  diskreq:=CreateStdIO(diskport);
  IF diskreq=NULL THEN
    DeletePort(diskport);
    RETURN FALSE;
  END;
  open:=OpenDevice("trackdisk.device",drive,diskreq,0);
 RETURN open=0;
END Init;

PROCEDURE Term;
BEGIN
  IF (open=0) THEN CloseDevice(diskreq); END;
  IF (diskreq<>NULL) THEN DeleteStdIO(diskreq); END;
  IF (diskport<>NULL) THEN DeletePort(diskport) END;
END Term;

PROCEDURE ShowFormat;
BEGIN
  WriteString(
    "Modula-2 TrackDisk Step Rate Utility V1 By David Q. King 3-Oct-87");
  WriteLn;
  WriteString("Format: DFx:/A n   (where 0<=x<=3 and 1200<=n<=40000)");
  WriteLn;
END ShowFormat;

PROCEDURE ParseArgs():BOOLEAN;
(*return false if there is nothing to do*)
BEGIN
  DontCare:=GetCL(argc, args); (* get arguments *)
  IF argc>0 THEN (* Parse Command Parameters to ticks and drive *)    
    IF Compare(args[0],"?")=Equal THEN
      RETURN FALSE;
    ELSE
      UpperCase(args[0]);
      IF Pos("DF0:DF1:DF2:DF3:",args[0],0,Where) THEN
        drive:= LONGCARD( Where DIV 4 );
        IF argc>1 THEN
          ConvertToCardinal(args[1], OK, Where);
          ticks:=LONGCARD(Where);
          IF OK THEN
            IF ticks<=1200 THEN
              ticks:=1200;
            END;
            IF ticks>=40000 THEN
              ticks:=0;  (* stupid input *)
            END;
          ELSE
            RETURN FALSE;
          END;
        END;
      END;
      RETURN TRUE; 
    END;
    ELSE
      RETURN FALSE;
  END; (* Parse Command Parameters *)
END ParseArgs;

BEGIN
  ticks:=0; drive:=0;
  IF ParseArgs() THEN
    IF Init() THEN
      ioUnitStuffPtr:=diskreq^.ioReq.ioUnit;  (* pointer to trackdisk
                                                 parameters *)
      INC(ioUnitStuffPtr, 44); (* pointer into private area
                                  of trackdisk! *)
      TrackTicks:=ioUnitStuffPtr; (* cost of doing business in Modula
                                     is that we have to do this for type
                                     compatibility.*)
      WriteString("Present delay count between steps is       ");
      WriteLongCard(TrackTicks^,8);
      WriteLn;
      IF ticks<>0 THEN (* change ticks *)
        TrackTicks^:=ticks;
        WriteString("Drive DF");WriteLongCard(drive,1);
        WriteString(": Step Rate delay count changed to");
        WriteLongCard(TrackTicks^,8);
        WriteString(" ticks."); WriteLn;
      END;
      Term;
    ELSE
      WriteString("Invalid arguments");
    END; (* if open... *)
    ELSE
      ShowFormat;
  END; (*good args*)
END Speedup.
